macroScript epp_macro category:"Piranesi" tooltip:"Create EPix Panorama" buttontext:"Make EPP"
(
    local cams = for cam in cameras \
        where (classof cam == Targetcamera  or  classof cam == Freecamera) \
        collect cam

    if (cams.count == 0) then
    (
        MessageBox "No cameras defined"
    )
    else
    (
        local stitch = getdir #scripts + "\utilities\stitch.exe"
        if (getfiles stitch).count == 0 then
        (
            messagebox ("Could not find " + stitch)
        )
        else
        (
            try(closerolloutfloater epprollout)
            catch()
            epprollout = newrolloutfloater "EPix Panorama" 200 277
            rollout params "Parameters"
            (
                dropdownlist cameralist "From Position of Camera"  items:(for cam in cams collect cam.name)
                spinner resol "Resolution" range:[10,2000,256] type:#integer
                button dorender "Create File"

                on dorender pressed do
                (
                    closerolloutfloater epprollout

                    local position = cams[cameralist.selection].position
                    local resolution = resol.value

                    local eppFile = getSaveFileName caption:"Save EPix Panorama" \
                        filename:(getDir #image + "\\" + (getFilenameFile maxFileName) ) \
                        types:"Piranesi EPix Panoramas (*.epp)|*.epp|All Files|*.*|"
                    if (eppFile != undefined) then  -- else user cancelled
                    (
                        local basefile = getFilenamePath(eppFile) + getFilenameFile(eppFile) + "_face"
                        local index = 0
                        local face = #()
                        for i=1 to 6 do
                        (
                            do
                            (
                                face[i] = (basefile + (index as String) + ".epx")
                                index += 1
                            )
                            while (getfiles face[i]).count != 0
                        )

                        local mycamera = Freecamera fov:(2*atan(resolution/(resolution-1.0)))
                        local rots=#((quat 0.7071067811865475244 0 0 0.7071067811865475244),
                                     (quat 0.5 -0.5 -0.5 0.5),
                                     (quat 0 0.7071067811865475244 0.7071067811865475244 0),
                                     (quat 0.5 0.5 0.5 0.5),
                                     (quat 1 0 0 0),
                                     (quat 0 0 0 1))
                        for i=1 to 6 do
                        (
                            mycamera.rotation=rots[i]
                            mycamera.position=position
                            print ("Rendering face " + (i as String) + " of 6")
                            render camera:mycamera outputwidth:resolution outputheight:resolution \
                                antialiasing:false outputfile:face[i] channels:#(#zDepth, #node, #normal) vfb:false
                        )
                        delete mycamera

                        print ("Combining faces")
                        shellLaunch  stitch \
                            (" \"" + eppfile + "\"" + \
                            " \"" + face[1] + "\"" + \
                            " \"" + face[2] + "\"" + \
                            " \"" + face[3] + "\"" + \
                            " \"" + face[4] + "\"" + \
                            " \"" + face[5] + "\"" + \
                            " \"" + face[6] + "\"" + \
                            " 0.00001" \
                        )
                    )
                )
            )
            addrollout params epprollout
        )
    )
)
